#!/usr/bin/php -f
<?php

#	-------------------------------- Version 2.02 ------------------------------------------------
#
#	Copyright 2007-2010 Arts Management Systems Ltd.   www.artsman.com
#
#	Feel free to modify this script to backup your postgres database
#
#	This is used to backup a postgres database.  It can be invoked from the 
#	terminal command, or auto-executed using a cron job.
#
#	This assumes that the script is running on the same machine as the database -AND-
#	that the localhost or 127.0.0.1/32 access in the pg_hba.conf file as 'trust' permissons
#
#	changes
#	091220	added date_default_timezone_set to remove some PHP messages on date function
#	091221	added vacuum analyse to the list of tasks that the backup does
#	100505	added mechanism to calculate the file size and stick in database
#	100614	fix issue with $filesize so that it works on backups larger that 2GB
#	101001	Change the backup script to postgres version 9.0
#	111106	Change the backup script to postgres version 9.1 
#
#	----------------------------------------------------------------------------------------------
#
#	To run from a terminal command, Type:
#			php /Users/Shared/Backups/backupTM.php
#
#	where pathname is the pathname to the backup file.
#
#	To run in a cron script so that it executed automatically, please see the web site
#	or look for a GUI tool called CRONIX that can help you set this up
#
#	Make the changes only
#	as indicated below.
#

#	Set the timezone of the computer that is running the backup otherwise it might assume
#	a setting that you don't like and generate warmning messages to the log.   Replace with one
#	of the following (or refer to http://www.php.net/manual/en/timezones.others.php)
#	AST,EST,CST, MST, PST  (or use daylite times, or 'Canada/Mountain'
#
#	comment out the 'date_default_timezone_set' if using old versons of PHP

date_default_timezone_set('MST');
#
#	Change  $backupDB ='xxxxx'; i nthe next line to be the name of the postgres database
#	that you want to backup

$backupDB = 'TheatreManagerDemo';

#	$localFolder indicates the name of the directory where the backups will be stored.   The postgres
#	user, and a minimum, will need read & write access to this directory.
#

$localFolder = '/Users/Shared/Backups';

#	The backup command for postgres needs to include the full pathname to the postgres pg_dump command.
#	the switches on the pg_dump command are:
#		-F c		Means to make a compressed backup file
#		-Z 9		is the compression level.   9 is the highest
#		-v			verbose mode ... show what is being dumped as it occurs.

$postgresBin = '/Library/PostgreSQL/9.1/bin/';
$backupCommand  = $postgresBin.'pg_dump -F c -Z 9 -v -U postgres ' . $backupDB;

#	The KeepFilesFor is used to determine how long to keep backup files for on the local machine
#	a good suggestion is to keep at least 30 days.     If this is set to zero, then no files will
#	be deleted during the backup process.   if it is set to a value, then as the file deletion
#	occurs, all files that are dated for the first of the month will also be kept forever.  This
#	is so that there is at least a monthly record of files.
#
#	if the backups occurr after midnight on the first of the month, then that effectively is an
#	automatic retention of monthly backups.    It is suggested that this script be set to run
#	just after midnight then.


$KeepFilesFor	= 30;

#	Optionally, specify an FTP site to send the file we just created to store it on another mahine
#	to store it off site.  If you put nothing for the $ftphost, an offsite backup will not occur
#	$ftpHost -> is the ip address or URL to send the database to.  eg:  myftpserver.org
#	$ftpUser -> is the user id to log into the remote FTP site with
#	$ftpPass -> is the password used by the FTP server to allow a login
#	$ftpFolder -> is the remote folder.    Examples are '/backup', or ''
#
#	$ftpDaysOldToPurge -> is the number of days to keep backup files on the FTP site.  They
#					will only be deleted if you are actually backup up the files remotely.  The
#					suggested default is 7 days.   Leave at zero
#					if you want the files to be there forever.
#
#	NOTE: FOR PCI COMPLIANCE, you will need to ensure that the data files are sent via a vpn
#	or secure tunnel if the data file is to leave your internal lan.  You will also need to
#	ensure that the offsite storage location and/or vendor is included in your PCI checklist
#	and adheres to PCI standards
#
#	If the $ftpHost is blank, then no backup will occur

$ftpHost  			= '';				#ftp site (Without ftp://) such as artsman.serveftp.org
$ftpUser			= 'userid';			#user id to log on to the FTP site
$ftpPass			= 'password';		#password to ftp site
$ftpFolder			= '';				#optional folder within the ftp site.  Must exist
$ftpDaysOldToPurge	= 7;

#	Optionally, specify a remote volume & folder if you want the backup script to send the backup file
#	to another machine.   This is not recommended because the mount must always be active on the current
#	machine AND the user must have permission to write to that other directory.
#
#	Since this is just a copy command, you could copy to another folder or disk on the same machine
#
#	set $remoteComputer to be something like
#	$remoteComputer="/Volumes/OtherServer/Users/Shared/Backup"
#

$remoteComputer='';


#-------------------------------------------------------------------------------------------------
#	change nothing below this line.    This first step backups up the database to a local
#	directory on the same machine
#

$backupFilename = $backupDB . '_TM_'. date('Ymd_His') . '.backup';

print(date('Y-m-d h:i a') . " Starting pg_dump of $backupDB\nBackup File Name:  $localFolder/$backupFilename\n");

system($backupCommand . ' > ' . $localFolder . '/' . $backupFilename);

# $filesize		= filesize ($localFolder . '/' . $backupFilename);  only works for files less that 2GB in size
$filesize		= sprintf("%u", filesize ($localFolder . '/' . $backupFilename));

print(date('Y-m-d h:i a') . ' Completed pg_dump. File size '.$filesize.'. Updating backup last time ...' . "\n");

#-------------------------------------------------------------------------------------------------
#
#	do not touch.  Update the last backup date/time after backup has run.
#
$updateCmd = "\"update fSystemPreferences set SPR_PRIOR_BACKUP_SIZE = SPR_LAST_BACKUP_SIZE,SPR_LAST_BACKUP_SIZE = abs(".$filesize."), SPR_DATE_LAST_BACKUP_SCRIPT_RUN = TIMESTAMP '".date('Y-m-d H:i:s')."';\"";
#	Note: remove -w if using postgres 8.3.x
system($postgresBin.'psql -U postgres -w -d '.$backupDB.' -c '.$updateCmd);

# and vacuum the database for performance
#
print(date('Y-m-d h:i a') . ' Starting vacuum analyse ...' . "\n");

system($postgresBin.'psql -U postgres -w -d '.$backupDB.' -c "vacuum analyse;"');

print(date('Y-m-d h:i a') . ' Completed vacuum analyse ...' . "\n");

system('du -sh ' . $localFolder . '/' . $backupFilename);

#----------------------------- This copies a file to a remote computer  ---------------------------
#	Optional step to copy the backup to another computers

if ($remoteComputer <> "")
	{
		print(date('Y-m-d h:i a') . " Attempting Copy to $remoteComputer\n");
		system('cp ' . $localFolder . '/' . $backupFilename .' ' . $remoteComputer);
	}
else
	{
		print(date('Y-m-d h:i a') . " Copy to remote computer *Disabled*\n");
	}
	

#---------------------------- This deletes files from local directory ----------------------------
# delete all files older than specified time, except for those created on the first of the month
# which will mean that there is a backup from the previous day, normaly done after midnight
# ... and we can keep those around forever.
#
# based on the unix 'find' command that looks like below.  Substitute  ls -l    for    rm -f
# if you want to simply list the files (near the end of the command.   Thanks to Unix online help.
#

if ($KeepFilesFor > 0)
	{
		print(date('Y-m-d h:i a') . " Purging local files older than $KeepFilesFor days\n");
		system('find ' . $localFolder . ' -type f \( -name "'. $backupDB . '*.backup" -not -name "' . $backupDB. '*01_*.backup" \) -mtime +'.$KeepFilesFor.' -exec rm -f {} \;');
	}
else
	{
		print(date('Y-m-d h:i a') . " Purging local files *disabled*\n");

	}

#-------------------------------------------------------------------------------------------------
#	quick fail if we are not doing any FTP activity

if ($ftpHost == "")
	{
		print(date('Y-m-d h:i a') . " Backups are done to local directory.  No FTP attempted\n");
		exit;
	}


#-------------------------------------------------------------------------------------------------
#	The optional step connects to a specified FTP site and moves a copy of the backup to the remote directory

$ftp = ftp_connect($ftpHost);

$login = ftp_login($ftp, $ftpUser, $ftpPass);

if (!$login)
	{
		print(date('Y-m-d h:i a') . " FTP connection has failed!\n");
		print(date('Y-m-d h:i a') . " Attempted to connect to $ftpHost as user $ftpUser\n");
		exit;
	}
else
	{
		print(date('Y-m-d h:i a') . " Connected to ftp://$ftpHost as user $ftpUser\n");
	}

$upload = ftp_put($ftp, 
$ftpFolder . '/' . $backupFilename, 
$localFolder . '/' . $backupFilename, 
FTP_BINARY); 

if (!$upload)
	{
		print(date('Y-m-d h:i a') . " FTP upload has failed!\n");
	}
else
	{
		print(date('Y-m-d h:i a') . " Uploaded $backupFilename to $ftpHost in $ftpFolder\n");
	}
	
#-------------------------------------------------------------------------------------------------
#	Optional step removes old files from the remote directory after $daysOldToPurge days elalpsed

$now = time();

if ($ftpDaysOldToPurge > 0)
	{	
	print("\n". date('Y-m-d h:i a') .' Checking for files '.$ftpDaysOldToPurge.' days old or older...'."\n\n");

	$fileList = ftp_nlist($ftp, $ftpFolder);

	while (list($null, $file) = each($fileList))
		{
			$age = round(($now - ftp_mdtm($ftp, $file)) / 86400, 2);
			print($file.' is '.$age.' days old -- ');

			if ($age >= $ftpDaysOldToPurge)
				{
					print('deleting!');
					ftp_delete($ftp, $file);
				}
			else
				{
					print('leaving!');
				}
			print("\n");
		}

	}
else
	{
		print("\n". date('Y-m-d h:i a') .' Purge of remote files *disabled*'."\n\n");

	}
	
#-------------------------------------------------------------------------------------------------
#	Close the FTP connection after upload and optional purge

ftp_close($ftp);

print(date('Y-m-d h:i a') . ' Finished backup and FTP process ...' . "\n");

?>
